@@ -17,6 +17,8 @@
 #include "term.h"
 #include "utils.h"
 #include "config.h"
+#include "cv.h"
+#include "exec.h"
 #include "globals.h"
 
 
@@ -24,33 +26,32 @@ void usage(void)
 {
 	printf("%s", version_str);
 	printf("\n\nmultitail [-cs|-Cs|-c-] [-i] inputfile [-i anotherinputfile] [...]\n");
-	printf("-l	parameter is a command to be executed\n");
-	printf("-L	see -l but add to the previous window\n");
+	printf("-i x	the following parameter is a filename (in case it starts with a dash)\n");
+	printf("-I x	like -i only this one merges this logfile into the previous window\n");
+	printf("-l x	parameter is a command to be executed\n");
+	printf("-L x	see -l but add to the previous window\n");
 	printf("-j	read from stdin (can be used (of course) only once)\n");
 	printf("-r interval    restart the command when it exited after `interval' seconds\n");
 	printf("-R interval    same as -r, only with this one only the difference is displayed\n");
-	printf("-i	the following parameter is a filename (in case it starts with a dash)\n");
+	printf("-Rc/-rc interval  like -r/-R but clean the window before each iteration\n");
 	printf("--cont         reconnect lines with a '\' at the end\n");
 	printf("--mergeall     merge all of the following files into the same window\n");
 	printf("--no-mergeall  stop merging all files into one window\n");
 	printf("--no-repeat    suppress repeating lines and replace them with a \"last message repeated x times\"\n");
-	printf("--mark-interval x\n");
-	printf("        when nothing comes in, print a '---mark---' line every 'x' seconds\n");
-	printf("--mark-change\n");
-	printf("        when multiple files are merged an multitail switches between two windows, print a markerline with the filename\n");
-	printf("--no-mark-change\n");
-	printf("        do NOT print the markerline when the file changes (overrides the configfile)\n");
+	printf("--mark-interval x	when nothing comes in, print a '---mark---' line every 'x' seconds\n");
+	printf("--mark-change	when multiple files are merged an multitail switches between two windows, print a markerline with the filename\n");
+	printf("--no-mark-change	do NOT print the markerline when the file changes (overrides the configfile)\n");
 	printf("-n x	initial number of lines to tail\n");
 	printf("-m x	set scrollback buffer size (# lines)\n");
 	printf("-mb x   set scrollback buffer size (in bytes, use xKB/MB/GB)\n");
 	printf("-bw a/f what to buffer: 'a'll or what went through the 'f'ilter\n");
-	printf("-I	see -i but add to the previous window\n");
 	printf("-a x	like 'tee': write (filtered) input to file 'x'\n");
 	printf("-A x	see -a: but write the unfiltered(!) input to file 'x'\n");
 	printf("-g x	redirect the input also (filtered) to command/process 'x'\n");
 	printf("-G x	redirect the unfiltered input also  to command/process 'x'\n");
-	printf("-q i path       path for new files with interval 'i', all in new windows\n");
-	printf("-Q i path       path for new files with interval 'i', all in the same (sub-)window\n");
+	printf("--label x	put in front of each line\n");
+	printf("-q i path       check every 'i' seconds for new files in 'path', create a new window for those\n");
+	printf("-Q i path       check every 'i' seconds for new files in 'path', put them all in the same window (using subwindows)\n");
 	printf("--closeidle x   close windows when more then 'x' seconds no new data was processed\n"); 
 	printf("--new-only	(for -q/-Q) only create windows for files created after multitail was started\n");
 	printf("-s x	vertical split screen (in 'x' columns)\n");
@@ -60,33 +61,33 @@ void usage(void)
 	printf("-S	prepend merged output with subwindow-number\n");
 	printf("-f	follow the following filename, not the descriptor\n");
 	printf("--retry	keep trying to open the following file if it is inaccessible\n");
+	printf("--retry-all like --retry but for all following files\n");
+	printf("-fr scheme  use the predefined filter from the configfile\n");
 	printf("-e[m]	print only when matching with this regexp\n");
 	printf("-ev	print only when NOT matching with this regexp\n");
 	printf("-ec	use regular expression but display the matches inverted on following file\n");
 	printf("-eC	use regexp, display everything but matches inverted on following file\n");
-	printf("-ex	execute command ('-ex regexp command') when matches\n");
+	printf("-ex	execute command ('-ex regexp command') when matches, matching line is given as commandline parameter\n");
+	printf("-eX     like -ex but only give the matching substring as commandline parameter to the command\n");
 	printf("-E	use regular expression on following files\n");
 	printf("-Ec	use regular expression but display the matches inverted on following files\n");
 	printf("-EC	use regexp, display everything but matches inverted on following files\n");
 	printf("-ke x   strip parts of the input using regular expression 'x'\n");
 	printf("-kr x y strip parts of the input starting at offset x and ending (not including!) offset y\n");
 	printf("-kc x y strip parts of the input: strip column 'y' with delimiter 'x'\n");
+	printf("-ks x   use edit scheme 'x' (defined in configfile)\n");
 	printf("-v      invert next regular expression (do not use with -ev/em)\n");
 	printf("-cv x	use conversion scheme 'x' (see multitail.conf)\n");
 	printf("-c	colorize current\n");
-	printf("-cS scheme\n");
-	printf("	use colorscheme 'scheme' (as defined in multitail.conf)\n");
+	printf("-cS scheme	use colorscheme 'scheme' (as defined in multitail.conf)\n");
 	printf("-csn    extra switch for the following switches; do not use reverse (inverted) colors\n");
 	printf("-Cs	colorize all following files with syslog-scheme\n");
 	printf("-C	colorize all following files\n");
-	printf("-Cf/-cf field delimiter\n");
-	printf("        colorize next/all file(s) depending on the given field number. fields\n");
-	printf("        are delimited with the given field-delimiter\n");
+	printf("-Cf/-cf field delimiter		colorize next/all file(s) depending on the given field number. fields are delimited with the given field-delimiter\n");
 	printf("-ci color  use 'color' (red, green, etc), usefull when merging multiple inputs\n");
 	printf("-c-	do NOT colorize the following file\n");
 	printf("-C-	do NOT colorize the following files\n");
-	printf("-cT term");
-	printf("	interpret terminal-codes from file/command (for terminal type 'term')\n");
+	printf("-cT term	interpret terminal-codes from file/command (for terminal type 'term')\n");
 	printf("-Z color   set color for markerline\n");
 	printf("-ts	add a timestamp (format configurable in multitail.conf) before each line\n");
 	printf("-T      put a timestamp in markerlines\n");
@@ -96,13 +97,20 @@ void usage(void)
 	printf("-z	do not \"window closed\" windows\n");
 	printf("-w	do not use colors\n");
 	printf("-u	set update interval (for slow links)\n");
-	printf("-p x [y]\n");
-	printf("	set linewrap (l=left/a=all/r=right/s=syslog,S=syslog w/o procname, o=offset -> 'y')\n");
+	printf("-p x [y]	set linewrap (l=left/a=all/r=right/s=syslog,S=syslog w/o procname,o=offset -> 'y',w=wordwrap)\n");
+	printf("-P      like -p but for all following files\n");
 	printf("-b n	set TAB-width\n");
 	printf("--basename	only display the filename (and not the path) in the statusline\n");
 	printf("-x str	switch on the xtermtitlebar stuff\n");
 	printf("-F file   use 'file' as configfile (instead of " CONFIG_FILE ")\n");
+	printf("--no-load-global-config do not read " CONFIG_FILE "\n");
+	printf("-o configfileparameter    do a setting which would normally be set in the configfile\n");
 	printf("-H x	show heartbeat (to keep your sessions alive)\n");
+	printf("-iw file i	check every 'i' seconds if 'file' appeared in the filesystem\n");
+	printf("-t x	display 'x' in the window-title (when MultiTail runs in an xterm)\n");
+	printf("--beep-interval x	beep every x lines processed\n");
+	printf("--bi x	like '--beep-interval' but only for current (sub-)window\n");
+	printf("-V	show version and exit\n");
 	printf("-h	this help\n");
 	printf("\n");
 	printf("You can have multiple regular expressions per file/command. Be warned: if\n");
@@ -114,91 +122,17 @@ void usage(void)
 	printf("%s\n", F1);
 }
 
-int get_value_arg(char *par, char *string, valcheck_t check)
-{
-	long int result;
-	int len, loop;
-	int multiplier = 1;
-
-	if (!string)
-		error_exit("%s needs a parameter\n", par);
-
-	len = strlen(string);
-
-	for(loop=0; loop<len; loop++)
-	{
-		if (!isdigit(string[loop]))
-		{
-			char dummy = toupper(string[loop]);
-
-			if (toupper(string[loop + 1]) == 'B' && (dummy == 'K' || dummy == 'M' || dummy == 'G'))
-			{
-				if (dummy == 'K')
-					multiplier = 1024;
-				else if (dummy == 'M')
-					multiplier = 1024 * 1024;
-				else if (dummy == 'G')
-					multiplier = 1024 * 1024 * 1024;
-
-				break;
-			}
-			else
-				error_exit("%s needs a value as parameter\n", par);
-		}
-	}
-
-	result = strtol(string, NULL, 10);
-
-	if (result > INT_MAX || result == LONG_MIN || result == LONG_MAX)
-		error_exit("value %s for parameter %s is too large\n", string, par);
-
-	result *= multiplier;
-
-	if (check == VAL_ZERO_POSITIVE)
-	{
-		if (result < 0)
-			error_exit("value for %s must be >= 0\n", par);
-	}
-	else if (check == VAL_POSITIVE_NOT_1)
-	{
-		if (result < 0 || result == 1)
-			error_exit("value for %s must be >= 0 and not 1\n", par);
-	}
-	else if (check == VAL_POSITIVE)
-	{
-		if (result <= 0)
-			error_exit("value for %s must be > 0\n", par);
-	}
-	else
-		error_exit("get_value_arg: internal error (%d)\n", check);
-
-	return (int)result;
-}
-
-int find_conversion_scheme(char *name)
-{
-	int loop;
-
-	for(loop=0; loop<n_conversions; loop++)
-	{
-		if (strcmp(conversions[loop].name, name) == 0)
-			return loop;
-	}
-
-	return -1;
-}
-
 void add_redir_to_file(char mode, char *file, redirect_t **predir, int *n_redirect)
 {
 	int cur_index = (*n_redirect)++;
-	*predir = (redirect_t *)myrealloc(*predir, (*n_redirect) * sizeof(redirect_t), "main: redirect_t-list");
+	*predir = (redirect_t *)myrealloc(*predir, (*n_redirect) * sizeof(redirect_t), "add_redir_to_file: redirect_t-list");
 
 	if (mode == 'a')
 		(*predir)[cur_index].type = REDIRECTTO_FILE_FILTERED;
 	else
 		(*predir)[cur_index].type = REDIRECTTO_FILE;
 
-	(*predir)[cur_index].redirect = mystrdup(file, "main: redirect destination");
+	(*predir)[cur_index].redirect = mystrdup(file, "add_redir_to_file: redirect destination");
 
 	(*predir)[cur_index].fd = open((*predir)[cur_index].redirect, O_WRONLY | O_CREAT | O_APPEND, S_IRUSR | S_IWUSR);
 	if ((*predir)[cur_index].fd == -1)
@@ -207,10 +141,10 @@ void add_redir_to_file(char mode, char *file, redirect_t **predir, int *n_redire
 
 void add_redir_to_proc(char mode, char *proc, redirect_t **predir, int *n_redirect)
 {
-	int fds[2];
+	int fds_to_proc[2], fds_from_proc[2];
 	int cur_index = (*n_redirect)++;
 
-	*predir = (redirect_t *)myrealloc(*predir, (*n_redirect) * sizeof(redirect_t), "main: redirect_t-list");
+	*predir = (redirect_t *)myrealloc(*predir, (*n_redirect) * sizeof(redirect_t), "add_redir_to_proc: redirect_t-list");
 
 	if ((*predir)[cur_index].type) error_exit("one can only set one redirection-type per (sub-)window\n");
 
@@ -219,36 +153,14 @@ void add_redir_to_proc(char mode, char *proc, redirect_t **predir, int *n_redire
 	else
 		(*predir)[cur_index].type = REDIRECTTO_PIPE;
 
-	(*predir)[cur_index].redirect = mystrdup(proc, "main: command to redirect to");
+	(*predir)[cur_index].redirect = mystrdup(proc, "add_redir_to_proc: command to redirect to");
 
-	if (pipe(fds) == -1)
-		error_exit("main: error creating pipe\n");
+	(*predir)[cur_index].pid = exec_with_pipe((*predir)[cur_index].redirect, fds_to_proc, fds_from_proc);
+	close(fds_to_proc[0]);
+	close(fds_from_proc[0]);
+	close(fds_from_proc[1]);
 
-	if (((*predir)[cur_index].pid = fork()) == -1)
-		error_exit("main: fork() failed\n");
-
-	if ((*predir)[cur_index].pid == 0)
-	{
-		int dummy_fd = open_null();
-
-		close(0);
-		if (dup(fds[0]) == -1) error_exit("add_redir_to_proc: dup() failed\n");
-		close(1);
-		close(2);
-		if (dup(dummy_fd) == -1) error_exit("add_redir_to_proc: dup() failed\n");
-		if (dup(dummy_fd) == -1) error_exit("add_redir_to_proc: dup() failed\n");
-		close(fds[1]);
-
-		/* start process */
-		if (-1 == execlp(shell, shell, "-c", (*predir)[cur_index].redirect, (void *)NULL)) error_exit("main: execlp of %s failed\n", (*predir)[cur_index].redirect);
-
-		/* if execlp returns, an error occured */
-		error_exit("main: error while starting process '%s'!\n", (*predir)[cur_index].redirect);
-	}
-
-	close(fds[0]);
-
-	(*predir)[cur_index].fd = fds[1];
+	(*predir)[cur_index].fd = fds_to_proc[1];
 }
 
 void argv_set_window_widths(char *widths)
@@ -266,7 +178,7 @@ void argv_set_window_widths(char *widths)
 		if (!pnt) break;
 
 		split++;
-		vertical_split = (int *)myrealloc(vertical_split, split * sizeof(int), "main: vertical split widths");
+		vertical_split = (int *)myrealloc(vertical_split, split * sizeof(int), "argv_set_window_widths: vertical split widths");
 
 		cur_width = get_value_arg("-sw", pnt, VAL_POSITIVE);
 		widths = NULL;
@@ -302,7 +214,7 @@ void argv_set_n_windows_per_column(char *pars)
 		if (!pnt) break;
 
 		index++;
-		n_win_per_col = (int *)myrealloc(n_win_per_col, index * sizeof(int), "main: vertical split n_win_per_col");
+		n_win_per_col = (int *)myrealloc(n_win_per_col, index * sizeof(int), "argv_set_n_windows_per_column: vertical split n_win_per_col");
 
 		cur_n = get_value_arg("-sn", pnt, VAL_POSITIVE);
 		pars = NULL;
@@ -316,7 +228,6 @@ void argv_set_n_windows_per_column(char *pars)
 
 int argv_add_re(char *mode, char *pars[], char invert_regex, re **pre, int *n_re, char *exprall)
 {
-	int rc;
 	char *cmd = NULL, *expr = NULL;
 	char regex_mode = 0;
 	int n_pars_used = 0;
@@ -334,8 +245,8 @@ int argv_add_re(char *mode, char *pars[], char invert_regex, re **pre, int *n_re
 		regex_mode = 'm';	/* m = match, only print when matches */
 	else if (mode[2] == 'v')
 		regex_mode = 'v';	/* v = !match, only print when not matching */
-	else if (mode[2] == 'x')
-		regex_mode = 'x';	/* x = execute */
+	else if (toupper(mode[2]) == 'X')
+		regex_mode = mode[2];	/* x = execute */
 	else
 		error_exit("%s is an unknown switch\n", mode);
 
@@ -343,24 +254,26 @@ int argv_add_re(char *mode, char *pars[], char invert_regex, re **pre, int *n_re
 	expr = pars[n_pars_used++];
 
 	/* and if there's anything to execute, get commandline */
-	if (regex_mode == 'x')
+	if (toupper(regex_mode) == 'X')
 	{
 		cmd = pars[n_pars_used++];
+
+		if (regex_mode == 'X' && (strchr(expr, '(') == NULL || strchr(expr, ')') == NULL))
+			error_exit("filterscheme rule: -eX requires a regular expression which selectes a substring using '(' and ')'\n");
 	}
 
 	/* compile & set expression */
 	/* allocate new structure */
-	*pre = (re *)myrealloc(*pre, sizeof(re) * (*n_re + 1), "main: list of regular expressions");
+	*pre = (re *)myrealloc(*pre, sizeof(re) * (*n_re + 1), "argv_add_re: list of regular expressions");
 
 	/* initialize structure */
 	memset(&(*pre)[*n_re], 0x00, sizeof(re));
 
 	/* compile */
-	if ((rc = regcomp(&(*pre)[*n_re].regex, expr, REG_EXTENDED)))
-		error_exit("failed to compile regular expression \"%s\": %s\n", expr, convert_regexp_error(rc, &(*pre)[*n_re].regex));
+	compile_re(&(*pre)[*n_re].regex, expr);
 
 	/* remember string for later edit */
-	(*pre)[*n_re].regex_str = mystrdup(expr, "main: -e regexp");
+	(*pre)[*n_re].regex_str = mystrdup(expr, "argv_add_re: -e regexp");
 
 	/* set flag on current file */
 	(*pre)[*n_re].use_regex = regex_mode;
@@ -378,7 +291,7 @@ int argv_add_re(char *mode, char *pars[], char invert_regex, re **pre, int *n_re
 
 	/* what to execute (if...) */
 	if (cmd)
-		(*pre)[*n_re].cmd = mystrdup(cmd, "main: -e cmd");
+		(*pre)[*n_re].cmd = mystrdup(cmd, "argv_add_re: -e cmd");
 	else
 		(*pre)[*n_re].cmd = NULL;
 
@@ -387,7 +300,7 @@ int argv_add_re(char *mode, char *pars[], char invert_regex, re **pre, int *n_re
 	return n_pars_used;
 }
 
-int argv_color_settings(char *mode, char *pars[], char *allcolor, char *curcolor, int *field_index, char **field_delimiter, myattr_t *cdef, term_t *cur_term_emul, int **cur_color_schemes, int *cur_n_color_schemes, myattr_t *alt_col_cdev1, myattr_t *alt_col_cdev2)
+int argv_color_settings(char *mode, char *pars[], char *allcolor, char *curcolor, int *field_index, char **field_delimiter, myattr_t *cdef, term_t *cur_term_emul, int_array_t *cur_color_schemes, myattr_t *alt_col_cdev1, myattr_t *alt_col_cdev2)
 {
 	int n_pars_used = 0;
 	char cur_mode = mode[2], doall = 0;
@@ -423,8 +336,6 @@ int argv_color_settings(char *mode, char *pars[], char *allcolor, char *curcolor
 		if (!cur_cscheme)
 			error_exit("%s requires a colorscheme-name\n", mode);
 
-		load_configfile(config_file);
-
 		if ((cur_scheme_index = find_colorscheme(cur_cscheme)) == -1)
 		{
 			if (use_colors)
@@ -433,7 +344,7 @@ int argv_color_settings(char *mode, char *pars[], char *allcolor, char *curcolor
 				error_exit("Colorschemes are not supported on monochrome terminals\n");
 		}
 
-		add_color_scheme(cur_color_schemes, cur_n_color_schemes, cur_scheme_index);
+		add_color_scheme(cur_color_schemes, cur_scheme_index);
 	}
 	else if (cur_mode == '-')	/* do not color current */
 	{
@@ -471,37 +382,45 @@ int argv_add_stripper(char *mode, char *pars[], strip_t **pstrip, int *n_strip)
 {
 	int n_pars_used = 0;
 
-	*pstrip = myrealloc(*pstrip, (*n_strip + 1) * sizeof(strip_t), "smain: trip_t");
-
 	if (mode[2] == 'e')
 	{
-		int rc;
 		char *re = pars[n_pars_used++];
 
+		*pstrip = myrealloc(*pstrip, (*n_strip + 1) * sizeof(strip_t), "argv_add_stripper: trip_t");
 		(*pstrip)[*n_strip].type = STRIP_TYPE_REGEXP;
-		(*pstrip)[*n_strip].regex_str = mystrdup(re, "smain: trip regexp string");
-		if ((rc = regcomp(&(*pstrip)[*n_strip].regex, re, REG_EXTENDED)))
-			error_exit("failed to compile regular expression \"%s\": %s\n", re, convert_regexp_error(rc, &(*pstrip)[*n_strip].regex));
+		(*pstrip)[*n_strip].regex_str = mystrdup(re, "argv_add_stripper: trip regexp string");
+		compile_re(&(*pstrip)[*n_strip].regex, re);
+		(*n_strip)++;
 	}
 	else if (mode[2] == 'r')
 	{
+		*pstrip = myrealloc(*pstrip, (*n_strip + 1) * sizeof(strip_t), "argv_add_stripper: trip_t");
 		(*pstrip)[*n_strip].type = STRIP_TYPE_RANGE;
 		(*pstrip)[*n_strip].start = get_value_arg(mode, pars[n_pars_used++], VAL_ZERO_POSITIVE);
 		(*pstrip)[*n_strip].end = get_value_arg(mode, pars[n_pars_used++], VAL_ZERO_POSITIVE);
 		if ((*pstrip)[*n_strip].end <= (*pstrip)[*n_strip].start)
 			error_exit("'-kr start end': end must be bigger then start\n");
+		(*n_strip)++;
 	}
 	else if (mode[2] == 'c')
 	{
+		*pstrip = myrealloc(*pstrip, (*n_strip + 1) * sizeof(strip_t), "argv_add_stripper: trip_t");
 		(*pstrip)[*n_strip].type = STRIP_TYPE_COLUMN;
-		(*pstrip)[*n_strip].del = mystrdup(pars[n_pars_used++], "smain: trip column delimiter");
+		(*pstrip)[*n_strip].del = mystrdup(pars[n_pars_used++], "argv_add_stripper: trip column delimiter");
 		(*pstrip)[*n_strip].col_nr = get_value_arg(mode, pars[n_pars_used++], VAL_ZERO_POSITIVE);
+		(*n_strip)++;
+	}
+	else if (mode[2] == 's')
+	{
+		char *scheme = pars[n_pars_used++];
+		int editscheme_index = find_editscheme(scheme);
+		if (editscheme_index == -1)
+			error_exit("-ks %s: scheme not found\n", scheme);
+		duplicate_es_array(pes[editscheme_index].strips, pes[editscheme_index].n_strips, pstrip, n_strip);
 	}
 	else
 		error_exit("'%s' is not recognized\n", mode);
 
-	(*n_strip)++;
-
 	return n_pars_used;
 }
 
@@ -524,18 +443,21 @@ void do_commandline(int argc, char *argv[])
 	char curcolor = 'n', allcolor = 'n';
 	int field_index = 0;
 	char *field_delimiter = NULL;
-	char follow_filename = 0, retry = 0, invert_regex = 0;
+	char follow_filename = -1, retry = 0, invert_regex = 0;
+	char retry_all = 0;
 	char exprall = 0;
 	int maxlines = -1;
 	char setallmaxlines = 0;
 	int restart = -1;
+	char restart_clear = 0;
 	char do_diff = 0;
-	char line_wrap = 'a';
-	int line_wrap_offset = 0;
+	char cur_line_wrap = -1;
+	int cur_line_wrap_offset = -1;
+	char all_line_wrap = 0;
+	char *label = NULL;
 	re *pre = NULL;
 	int n_re = 0;
-	int *cur_color_schemes = NULL;
-	int cur_n_color_schemes = 0;
+	int_array_t cur_color_schemes = { NULL, 0, 0 };
 	myattr_t cdef = { -1, -1 };
 	int window_height = -1;
 	int initial_n_lines_tail = -1;
@@ -558,10 +480,30 @@ void do_commandline(int argc, char *argv[])
 	char cont = 0;
 	char marker_of_other_window = 0;
 	char no_marker_of_other_window = 0;
-	char bufferwhat = default_bufferwhat;
+	char bufferwhat = -1;
+	int cur_beep_interval = -1;
 
 	char do_add_timestamp = 0;
-	int conversion = -1;
+	int_array_t conversions = { NULL, 0, 0 };
+
+	/* first, before we load the main configfile, see if we should load the global
+	 * file or not
+	 */
+	for(loop=1; loop<argc; loop++)
+	{
+		if (strcmp(argv[loop], "--no-load-global-config") == 0)
+		{
+			load_global_config = 0;
+			break;
+		}
+	}
+	if (load_global_config)
+		(void)load_configfile(NULL);
+	/* set defaults from configfile */
+	bufferwhat = default_bufferwhat;
+	cur_line_wrap = default_linewrap;
+	cur_line_wrap_offset = default_line_wrap_offset;
+	follow_filename = default_follow_filename;
 
 	/* parse commandline */
 	for(loop=1; loop<argc; loop++)
@@ -572,6 +514,10 @@ void do_commandline(int argc, char *argv[])
 
 			exit(0);
 		}
+		else if (strcmp(argv[loop], "--no-load-global-config") == 0)
+		{
+			/* ignore */
+		}
 		else if (strcmp(argv[loop], "-bw") == 0)
 		{
 			char *what = argv[++loop];
@@ -646,6 +592,8 @@ void do_commandline(int argc, char *argv[])
 			config_file = argv[++loop];
 			if (file_exist(config_file) == -1)
 				error_exit("Config-file %s does not exist\n", config_file);
+
+			(void)do_load_config(NULL, config_file);
 		}
 		else if (strcmp(argv[loop], "-Z") == 0)
 		{
@@ -673,15 +621,19 @@ void do_commandline(int argc, char *argv[])
 				error_exit("-x requires a parameter\n");
 			set_title = mystrdup(argv[loop], "-x");
 		}
-		else if (strcmp(argv[loop], "-p") == 0)
+		else if (argv[loop][0] == '-' && toupper(argv[loop][1]) == 'P')
 		{
 			char *lw = argv[++loop];
 			if (lw)
 			{
-				line_wrap = lw[0];
-				if (line_wrap == 'o')
-					line_wrap_offset = get_value_arg("-p", argv[++loop], VAL_ZERO_POSITIVE);
-				else if (line_wrap != 'a' && line_wrap != 'l' && line_wrap != 'r' && toupper(line_wrap) != 'S')
+				if (argv[loop - 1][1] == 'P')
+					all_line_wrap = 1;
+				else
+					all_line_wrap = 0;
+				cur_line_wrap = lw[0];
+				if (cur_line_wrap == 'o')
+					cur_line_wrap_offset = get_value_arg("-p", argv[++loop], VAL_ZERO_POSITIVE);
+				else if (cur_line_wrap != 'a' && cur_line_wrap != 'l' && cur_line_wrap != 'r' && toupper(cur_line_wrap) != 'S' && cur_line_wrap != 'w')
 					error_exit("Invalid mode fore -p\n");
 			}
 			else
@@ -691,6 +643,11 @@ void do_commandline(int argc, char *argv[])
 		{
 			retry = 1;
 		}
+		else if (strcmp(argv[loop], "--retry-all") == 0)
+		{
+			retry_all = 1;
+			retry = 1;
+		}
 		else if (strcmp(argv[loop], "-n") == 0)
 		{
 			initial_n_lines_tail = get_value_arg("-n", argv[++loop], VAL_ZERO_POSITIVE);
@@ -703,11 +660,14 @@ void do_commandline(int argc, char *argv[])
 		{
 			update_interval = get_value_arg("-u", argv[++loop], VAL_ZERO_POSITIVE);
 		}
-		else if (strcasecmp(argv[loop], "-r") == 0)
+		else if (argv[loop][0] == '-' && toupper(argv[loop][1]) == 'R')
 		{
 			if (argv[loop][1] == 'R')
 				do_diff = 1;
 
+			if (argv[loop][2] == 'c')
+				restart_clear = 1;
+
 			restart = get_value_arg("-r/R", argv[++loop], VAL_ZERO_POSITIVE);
 		}
 		else if (strcmp(argv[loop], "-s") == 0)
@@ -726,13 +686,44 @@ void do_commandline(int argc, char *argv[])
 		{
 			window_height = get_value_arg("-wh", argv[++loop], VAL_POSITIVE);
 		}
-		else if (strcmp(argv[loop], "-cv") == 0) /* FIXME: multiple */
+		else if (strcmp(argv[loop], "-fr") == 0)
 		{
-			load_configfile(config_file);
+			char *par = argv[++loop];
+			for(;;)
+			{
+				int filter;
+				char *komma = strchr(par, ',');
+				if (komma)
+					*komma = 0x00;
 
-			conversion = find_conversion_scheme(argv[++loop]);
-			if (conversion == -1)
-				error_exit("'%s' is an unknown conversionscheme\n", argv[loop]);
+				filter = find_filterscheme(par);
+				if (filter == -1)
+					error_exit("'%s' is an unknown filter scheme\n", par);
+
+				duplicate_re_array(pfs[filter].pre, pfs[filter].n_re, &pre, &n_re);
+
+				if (!komma)
+					break;
+
+				par = komma + 1;
+			}
+		}
+		else if (strcmp(argv[loop], "-cv") == 0)
+		{
+			char *par = argv[++loop];
+			for(;;)
+			{
+				char *komma = strchr(par, ',');
+				if (komma)
+					*komma = 0x00;
+
+				add_conversion_scheme(&conversions, par);
+
+				if (!komma)
+					break;
+
+				par = komma + 1;
+			}
 		}
 		else if (argv[loop][0] == '-' && toupper(argv[loop][1]) == 'E')
 		{
@@ -748,7 +739,7 @@ void do_commandline(int argc, char *argv[])
 		}
 		else if (argv[loop][0] == '-' && toupper(argv[loop][1]) == 'C')
 		{
-			loop += argv_color_settings(argv[loop], &argv[loop + 1], &allcolor, &curcolor, &field_index, &field_delimiter, &cdef, &cur_term_emul, &cur_color_schemes, &cur_n_color_schemes, &alt_col_cdev1, &alt_col_cdev2);
+			loop += argv_color_settings(argv[loop], &argv[loop + 1], &allcolor, &curcolor, &field_index, &field_delimiter, &cdef, &cur_term_emul, &cur_color_schemes, &alt_col_cdev1, &alt_col_cdev2);
 		}
 		else if (strcmp(argv[loop], "-f") == 0)
 		{
@@ -774,6 +765,10 @@ void do_commandline(int argc, char *argv[])
 				setallmaxbytes = 1;
 			maxbytes = get_value_arg("-m/M", argv[++loop], VAL_ZERO_POSITIVE);
 		}
+		else if (strcasecmp(argv[loop], "--label") == 0)
+		{
+			label = argv[++loop];
+		}
 		else if (strcasecmp(argv[loop], "-i") == 0 || argv[loop][0] != '-' ||
 				strcasecmp(argv[loop], "-l") == 0 ||
 				strcasecmp(argv[loop], "-j") == 0 ||
@@ -788,8 +783,6 @@ void do_commandline(int argc, char *argv[])
 			int check_interval = 0;
 			proginfo *cur;
 
-			load_configfile(config_file);
-
 			if (strcasecmp(argv[loop], "-l") == 0)
 			{
 				is_cmd = 1;
@@ -797,7 +790,7 @@ void do_commandline(int argc, char *argv[])
 
 			if (strcasecmp(argv[loop], "-j") == 0)
 			{
-				if (used_stdin == MY_TRUE) error_exit("One can use -j only once!\n");
+				if (used_stdin == MY_TRUE) error_exit("One can use %s only once!\n", argv[loop]);
 				is_stdin = used_stdin = MY_TRUE;
 			}
 
@@ -844,7 +837,7 @@ void do_commandline(int argc, char *argv[])
 					cur = cur -> next;
 				}
 
-				cur -> next = (proginfo *)mymalloc(sizeof(proginfo), "main: proginfo");
+				cur -> next = (proginfo *)mymalloc(sizeof(proginfo), "do_commandline: proginfo");
 
 				cur = cur -> next;
 
@@ -852,11 +845,11 @@ void do_commandline(int argc, char *argv[])
 			}
 			else
 			{
-				pi = (proginfo *)myrealloc(pi, (nfd + 1) * sizeof(proginfo), "pmain: roginfo");
+				pi = (proginfo *)myrealloc(pi, (nfd + 1) * sizeof(proginfo), "do_commandline: roginfo");
 
-				lb = (buffer *)myrealloc(lb, (nfd + 1) * sizeof(buffer), "bmain: uffers");
+				lb = (buffer *)myrealloc(lb, (nfd + 1) * sizeof(buffer), "do_commandline: uffers");
 
-				nsubwindows = (char *)myrealloc(nsubwindows, (nfd + 1) * sizeof(char), "main: subwindows");
+				nsubwindows = (char *)myrealloc(nsubwindows, (nfd + 1) * sizeof(char), "do_commandline: subwindows");
 				nsubwindows[nfd] = 1;
 				memset(&lb[nfd], 0x00, sizeof(buffer));
 
@@ -892,11 +885,11 @@ void do_commandline(int argc, char *argv[])
 			if (is_stdin == MY_FALSE)
 			{
 				if (!dummy) error_exit("No filename given!\n");
-				cur -> filename = mystrdup(dummy, "main: cur -> filename");
+				cur -> filename = mystrdup(dummy, "do_commandline: cur -> filename");
 			}
 			else
 			{
-				cur -> filename = mystrdup("STDIN", "main: read from stdin");
+				cur -> filename = mystrdup("STDIN", "do_commandline: read from stdin");
 			}
 			cur -> is_command = is_cmd;
 			cur -> check_interval = check_interval;
@@ -917,8 +910,9 @@ void do_commandline(int argc, char *argv[])
 			window_height = -1;
 
 			/* store regular expression(s) */
-			cur -> n_re = n_re;
-			cur -> pre = pre;
+			cur -> pre = NULL;
+			cur -> n_re = 0;
+			duplicate_re_array(pre, n_re, &cur -> pre, &cur -> n_re);
 			if (exprall == 0)
 			{
 				n_re = 0;
@@ -938,81 +932,83 @@ void do_commandline(int argc, char *argv[])
 			pstrip = NULL;
 			n_strip = 0;
 
-			cur -> conversion_index = conversion;
-			conversion = -1;
+			cur -> conversions = conversions;
+			init_iat(&conversions);
 
 			/* line wrap */
-			cur -> line_wrap = line_wrap;
-			line_wrap = 'a';
-			cur -> line_wrap_offset = line_wrap_offset;
+			cur -> line_wrap = cur_line_wrap;
+			cur -> line_wrap_offset = cur_line_wrap_offset;
+			if (!all_line_wrap)
+			{
+				cur_line_wrap = default_linewrap;
+				cur_line_wrap_offset = default_line_wrap_offset;
+			}
 
 			cur -> retry_open = retry;
 			cur -> follow_filename = follow_filename;
+			follow_filename = default_follow_filename;
 
 			cur -> cont = cont;
 			cont = 0;
 
 			/* 'watch' functionality configuration (more or less) */
-			cur -> restart = restart;
+			cur -> restart.restart = restart;
+			cur -> restart.restart_clear = restart_clear;
 			restart = -1; /* invalidate for next parameter */
-			cur -> first = 1;
-			cur -> do_diff = do_diff;
+			restart_clear = 0;
+			cur -> restart.first = 1;
+			cur -> restart.do_diff = do_diff;
 			do_diff = 0;
 
-			cur -> suppress_repeating_lines = no_repeat;
+			cur -> repeat.suppress_repeating_lines = no_repeat;
 
 			cur -> mark_interval = mark_interval;
 
 			/*  colors */
-			cur -> attributes = cdef;
-			cur -> alt_col_cdev1 = alt_col_cdev1;
-			cur -> alt_col_cdev2 = alt_col_cdev2;
-			cur -> syslog_noreverse = syslog_noreverse;
+			cur -> cdef.attributes = cdef;
+			cur -> cdef.alt_col_cdev1 = alt_col_cdev1;
+			cur -> cdef.alt_col_cdev2 = alt_col_cdev2;
+			cur -> cdef.syslog_noreverse = syslog_noreverse;
 			if (curcolor != 'n' && curcolor != 0)
 			{
-				cur -> colorize = curcolor;
-				cur -> color_schemes = cur_color_schemes;
-				cur -> n_color_schemes = cur_n_color_schemes;
-				cur_color_schemes = NULL;
-				cur_n_color_schemes = 0;
+				cur -> cdef.colorize = curcolor;
+				cur -> cdef.color_schemes = cur_color_schemes;
+				init_iat(&cur_color_schemes);
 			}
 			else
 			{
 				if (allcolor == 'n' && default_color_scheme != -1)
 				{
-					cur -> colorize = 'S';
-					add_color_scheme(&cur -> color_schemes, &cur -> n_color_schemes, default_color_scheme);
+					cur -> cdef.colorize = 'S';
+					add_color_scheme(&cur -> cdef.color_schemes, default_color_scheme);
 				}
 				else if (allcolor == 'n')
 				{
-					cur -> colorize = 0;
+					cur -> cdef.colorize = 0;
 				}
 				else if (allcolor == 'S')
 				{
-					int n_bytes = sizeof(int) * cur_n_color_schemes;
-
-					cur -> colorize = 'S';
-					cur -> n_color_schemes = cur_n_color_schemes;
-					cur -> color_schemes = mymalloc(n_bytes, "allcolor: cur -> color_schemes");
-					memcpy(cur -> color_schemes, cur_color_schemes, n_bytes);
+					cur -> cdef.colorize = 'S';
+					cur -> cdef.color_schemes = cur_color_schemes;
 				}
 				else
 				{
-					cur -> colorize = allcolor;
+					cur -> cdef.colorize = allcolor;
 				}
 			}
-			cur -> field_nr = field_index;
-			cur -> field_del = field_delimiter ? mystrdup(field_delimiter, "main: cur -> field_del") : NULL;
+			cur -> cdef.field_nr = field_index;
+			cur -> cdef.field_del = field_delimiter ? mystrdup(field_delimiter, "do_commandline: cur -> field_del") : NULL;
 
-			cur -> sccfirst = 1;
+			cur -> statistics.sccfirst = 1;
 
 			curcolor = 'n';
-			retry = follow_filename = 0;
+			if (!retry_all)
+				retry = 0;
 
 			if (is_giw)	/* skip over the checkinterval */
 				loop++;
 
-			cur -> term_emul = cur_term_emul;
+			cur -> cdef.term_emul = cur_term_emul;
 			cur_term_emul = TERM_IGNORE;
 
 			/* redirect input also to a file or pipe */
@@ -1020,6 +1016,12 @@ void do_commandline(int argc, char *argv[])
 			n_redirect = 0;
 			cur -> predir = predir;
 			predir = NULL;
+
+			cur -> beep.beep_interval = cur_beep_interval;
+			cur_beep_interval = -1;
+
+			cur -> label = label;
+			label = NULL;
 		}
 		else if (strcmp(argv[loop], "-du") == 0)
 		{
@@ -1063,20 +1065,35 @@ void do_commandline(int argc, char *argv[])
 
 			add_glob_check(check_glob, check_interval, merge, new_only);
 		}
-		else if (strcasecmp(argv[loop], "--mark-change") == 0)
+		else if (strcmp(argv[loop], "--mark-change") == 0)
 		{
 			marker_of_other_window = 1;
 		}
-		else if (strcasecmp(argv[loop], "--no-mark-change") == 0)
+		else if (strcmp(argv[loop], "--no-mark-change") == 0)
 		{
 			no_marker_of_other_window = 1;
 		}
+		else if (strcmp(argv[loop], "-o") == 0)
+		{
+			loop++;
+
+			if (!argv[loop])
+				error_exit("-o requires a parameter\n");
+			else
+				config_file_entry(argv[loop]);
+		}
+		else if (strcmp(argv[loop], "--beep-interval") == 0)
+		{
+			beep_interval = get_value_arg("--beep-interval", argv[++loop], VAL_POSITIVE);
+		}
+		else if (strcmp(argv[loop], "--bi") == 0)
+		{
+			cur_beep_interval = get_value_arg("--bi", argv[++loop], VAL_POSITIVE);
+		}
 		else
 		{
 			if (strcmp(argv[loop], "-h") != 0)
-			{
 				fprintf(stderr, "unknown parameter '%s'\n", argv[loop]);
-			}
 
 			usage();
 
