@@ -23,7 +23,8 @@ extern char tab_width;
 extern time_t mt_started;
 extern int *vertical_split;
 extern int *n_win_per_col;
-extern int terminal_index;
+extern proginfo *terminal_index;
+extern int terminal_main_index;
 extern char prev_term_char;
 extern int n_keybindings;
 extern keybinding *keybindings;
@@ -81,12 +82,37 @@ extern char idleline_char;
 extern char msgline_char;
 extern char *replace_by_markerline;
 extern char default_bufferwhat;
-
 extern color_scheme *cschemes;
 extern int n_cschemes;
 extern colorpairs cp;
-
 extern pid_t children_list[MAX_N_SPAWNED_PROCESSES];
 extern int n_children;
-
 extern term_t term_type;
+extern regex_t global_highlight_re;
+extern char *global_highlight_str;
+extern mybool_t re_case_insensitive;
+extern filterscheme *pfs;
+extern int n_fs;
+extern char load_global_config;
+extern char default_linewrap;
+extern int default_line_wrap_offset;
+extern char default_follow_filename;
+extern char do_not_close_closed_windows;
+extern char suppress_empty_lines;
+extern editscheme *pes;
+extern int n_es;
+extern char splitline_mode;
+extern myattr_t splitline_attrs;
+extern int inverse_attrs;
+extern char abort_key;
+extern char exit_key;
+extern int beep_interval;
+extern int linecounter_for_beep;
+extern int did_n_beeps;
+extern char *defaultcscheme;
+extern char *line_ts_format;
+extern int default_min_shrink;
+extern char default_sb_showwinnr;
+extern int wordwrapmaxlength;
+extern history_t search_h;
+extern history_t cmdfile_h;
