@@ -8,6 +8,7 @@
 #include "help.h"
 #include "term.h"
 #include "utils.h"
+#include "globals.h"
 
 char *help_main_menu[] = {
 		"MultiTail lets you create one or more windows in",
@@ -23,6 +24,12 @@ char *help_main_menu[] = {
 		"One can press the following keys:",
 		" ^q^     quit the program",
 		" ^F1^    this help",
+		" ^/^     search in all windows",
+		" ^shift^ + ^/^  highlight in all windows",
+		" ^b^     scroll back",
+		" ^B^     scroll back in ALL windows merged into",
+		"       one window",
+		" ^I^     toggle case sensitivity for searches",
 		" ^a^     add a new window or add another file/",
 		"       program to an existing window",
 		" ^d^     delete a window (or a file/program from a",
@@ -38,6 +45,9 @@ char *help_main_menu[] = {
 		" ^0^...^9^ set a mark in a window so that you can",
 		"       easily see what has changed since the last",
 		"       time you looked",
+		" ^R^     restart a window",
+		" ^ctrl^ + ^v^  select conversion scheme(s)",
+		" ^ctrl^ + ^t^  toggle subwindow before lines",
 		" ^Y^     send a signal to a window (for commands)",
 		" ^y^     set linewrap",
 		" ^o^     wipe (clear) a window",
@@ -45,6 +55,7 @@ char *help_main_menu[] = {
 		" ^T^     truncate a logfile",
 		" ^g^     make screendump",
 		" ^r^     redraw screen",
+		" ^I^     toggle case sensitivity for searches",
 		" ^l^     list keybindings",
 		" ^i^     info",
 		" ^t^     statistics",
@@ -56,6 +67,7 @@ char *help_main_menu[] = {
 		"       with the current settings (the files that",
 		"       are monitored, regular expressions, etc.)",
 		" ^n^     clear the buffer of a window",
+		" ^N^     clear all buffers of all windows",
 		" ^m^     set the size of a buffer",
 		" ^k^     enter a (limited) terminal-mode",
 		" ^p^     pause all windows",
@@ -294,6 +306,10 @@ char *help_regexp_menu[] = {
 		"       of 10 per file/command)",
 		" ^e^     edit an existing regexp",
 		" ^d^     delete one of the existing regexps",
+		" ^D^     move an entry down, this also changes",
+		"       priority of this reg.exp.",
+		" ^U^     move an entry up",
+		" ^r^     reset counter",
 		" ^q^/^x^   leave this menu",
 		NULL
 	};
@@ -581,9 +597,13 @@ char *help_scrollback_help[] = {
 		" ^c^     set colors",
 		" ^f^/^/^   search for a string in the buffer",
 		" ^n^     find the next occurence",
-		" ^y^     toggle linewrap. if linewrap is disabled,",
+		" ^Y^     toggle linewrap. if linewrap is disabled,",
 		"       one can scroll left/right with the cursor-",
 		"       keys.",
+		" ^ctrl^+^t^ show (sub-)window number in front of",
+		"       each line",
+		" ^t^     show a list of window-nrs and what they",
+		"       contain. can also display statistics.",
 		" ^s^     save the buffer to a file",
 		"Press ^ctrl+g^ to abort.",
 		NULL
@@ -865,8 +885,64 @@ char *help_edit_color_change_name[] = {
 	};
 
 char *help_hide_but_window[] = {
+		"Select which window to keep open. All other",
+		"windows will be closed.",
+		NULL
+	};
+
+char *help_search_in_all_windows[] = {
+		"This functionality lets you search in all windows.",
+		"The result will be merged into one new window.",
+		NULL
+	};
+
+char *help_highlight_in_all_windows[] = {
+		"This functionality lets you search in all windows.",
+		"All lines with the search-string will be",
+		"highlighted (displayed in reverse).",
+		NULL
+	};
+
+char *help_select_conversionschemes[] = {
+		"With a conversionscheme one can let MultiTail",
+		"automatically convert e.g. ip-addresses, time-",
+		"stamps, etc. to something more readable. In this",
+		"menu you can select what conversionscheme(s) to",
+		"apply to a (sub-)window.",
+		NULL
+	};
+
+char *help_select_restart_window[] = {
+		"Restarting a window stops- and restarts the tail-",
+		"or command running underneath a (sub-)window.",
+		NULL
+	};
+
+char *help_select_restart_window_all[] = {
+		"This chooses wether to select all sub-windows or",
+		"just one specific.",
+		NULL
+	};
+
+char *help_select_terminal_emulation[] = {
+		"MultiTail can emulate a terminal. That way it can",
+		"display color (escape-)codes that sometimes are",
+		"in (log-)files.",
+		NULL
+	};
+
+char *help_clear_buffer[] = {
+		"Select window for which to clear the scrollback",
+		"buffer.",
+		NULL
+	};
+
+char *help_history[] = {
 		/*2345678901234567890123456789012345678901234567890 */
-		"FIXME", /* FIXME */
+		"Select a string from this list. These strings have",
+		"been used in the past in MultiTail. They are",
+		"stored in a file. Check the 'history_file' entry",
+		"in the configuration-file to see in what file.",
 		NULL
 	};
 
@@ -900,7 +976,6 @@ help_t	help[] =
 	{ HELP_ENTER_REGEXP_SELECT_SUBWINDOW,	help_enter_regexp_select_subwindow },
 	{ HELP_ENTER_STRIPPER_SELECT_SUBWINDOW,	help_enter_stripper_select_subwindow },
 	{ HELP_REGEXP_MENU,			help_regexp_menu },
-
 	{ HELP_ENTER_REGEXP,			help_enter_regexp },
 	{ HELP_NEGATE_REGEXP,			help_negate_regexp },
 	{ HELP_REGEXP_USAGE,			help_regexp_usage },
@@ -967,6 +1042,14 @@ help_t	help[] =
 	{ HELP_EDIT_COLOR,			help_edit_color },
 	{ HELP_EDIT_COLOR_CHANGE_NAME,		help_edit_color_change_name },
 	{ HELP_HIDE_BUT_WINDOW,			help_hide_but_window },
+	{ HELP_SEARCH_IN_ALL_WINDOWS,		help_search_in_all_windows },
+	{ HELP_HIGHLIGHT_IN_ALL_WINDOWS,	help_highlight_in_all_windows },
+	{ HELP_SELECT_CONVERSIONSCHEMES,	help_select_conversionschemes },
+	{ HELP_SELECT_RESTART_WINDOW,		help_select_restart_window },
+	{ HELP_SELECT_RESTART_WINDOW_ALL,	help_select_restart_window_all },
+	{ HELP_SELECT_TERMINAL_EMULATION,	help_select_terminal_emulation },
+	{ HELP_CLEAR_BUFFER,			help_clear_buffer },
+	{ HELP_HISTORY,				help_history },
 	{ -1, NULL }
 };
 
@@ -1036,7 +1119,7 @@ void show_help(int what_help)
 			{
 				pos -= 15;
 			}
-			else if ((c == KEY_NPAGE || c == ' ') && pos < (line_cnt - 1))
+			else if ((c == KEY_NPAGE || c == ' ') && (pos + 15) < (line_cnt - 1))
 			{
 				pos += 15;
 			}
@@ -1044,7 +1127,7 @@ void show_help(int what_help)
 			{
 				pos = 0;
 			}
-			else if (tolower(c) == 'q' || c == 7)
+			else if (tolower(c) == 'q' || c == abort_key)
 			{
 				break;
 			}
