@@ -15,14 +15,14 @@
 #include "globals.h"
 
 
-int edit_stripper_edit_regexp(NEWWIN *win, char **old_re)
+int edit_stripper_edit_regexp(NEWWIN *win, char **old_re, mybool_t *case_insensitive)
 {
 	int changed = 0;
 	char *new_re;
 
 	/* ask new reg exp */
 	mvwprintw(win -> win, 15, 2, "Edit regular expression:          ");
-	new_re = edit_string(win, 16, 2, 58, 128, 0, *old_re, HELP_ENTER_REGEXP, -1);
+	new_re = edit_string(win, 16, 2, 58, 128, 0, *old_re, HELP_ENTER_REGEXP, -1, &search_h, case_insensitive);
 
 	if (new_re)
 	{
@@ -45,7 +45,7 @@ int edit_stripper_edit_offsets(NEWWIN *win, int *offset_start, int *offset_end)
 		snprintf(buffer, sizeof(buffer), "%d", *offset_start);
 
 	mvwprintw(win -> win, 15, 2, "Enter start offset:          ");
-	str_start = edit_string(win, 16, 2, 5, 5, 1, buffer, HELP_STRIPPER_START_OFFSET, -1);
+	str_start = edit_string(win, 16, 2, 5, 5, 1, buffer, HELP_STRIPPER_START_OFFSET, -1, NULL, NULL);
 	if (str_start)
 	{
 		if (*offset_end != -1)
@@ -57,7 +57,7 @@ int edit_stripper_edit_offsets(NEWWIN *win, int *offset_start, int *offset_end)
 		memset(linebuf, ' ', sizeof(linebuf));
 		linebuf[sizeof(linebuf) - 1] = 0x00;
 		mvwprintw(win -> win, 16, 1, "%s", linebuf);
-		str_end = edit_string(win, 16, 2, 5, 5, 1, buffer, HELP_STRIPPER_END_OFFSET, -1);
+		str_end = edit_string(win, 16, 2, 5, 5, 1, buffer, HELP_STRIPPER_END_OFFSET, -1, NULL, NULL);
 		if (str_end)
 		{
 			*offset_start = atoi(str_start);
@@ -82,7 +82,7 @@ int edit_stripper_edit_field(NEWWIN *win, char **del, int *col_nr)
 	char linebuf[58 + 1];
 
 	mvwprintw(win -> win, 15, 2, "Enter delimiter:              ");
-	new_del = edit_string(win, 16, 2, 58, 128, 0, *del, HELP_STRIPPER_DELIMITER, -1);
+	new_del = edit_string(win, 16, 2, 58, 128, 0, *del, HELP_STRIPPER_DELIMITER, -1, NULL, NULL);
 	if (new_del)
 	{
 
@@ -94,7 +94,7 @@ int edit_stripper_edit_field(NEWWIN *win, char **del, int *col_nr)
 		if (*col_nr != -1)
 			snprintf(buffer, sizeof(buffer), "%d", *col_nr);
 
-		new_col_nr = edit_string(win, 16, 2, 5, 5, 1, buffer, HELP_STRIPPER_COL_NR, -1);
+		new_col_nr = edit_string(win, 16, 2, 5, 5, 1, buffer, HELP_STRIPPER_COL_NR, -1, NULL, NULL);
 	}
 
 	if (new_del != NULL && new_col_nr != NULL)
@@ -117,6 +117,7 @@ int edit_strippers(void)
 	proginfo *cur;
 	int f_index = 0;
 	int cur_strip = 0;
+	mybool_t case_insensitive = re_case_insensitive;
 
 	/* select window */
 	if (nfd > 1)
@@ -133,7 +134,7 @@ int edit_strippers(void)
 		return 0;
 
 	/* create window */
-	mywin = create_popup(24, 70);
+	mywin = create_popup(23, 70);
 
 	for(;;)
 	{
@@ -150,13 +151,13 @@ int edit_strippers(void)
 		/* clear */
 		memset(linebuf, ' ', sizeof(linebuf) - 1);
 		linebuf[sizeof(linebuf) - 1] = 0x00;
-		for(loop=4; loop<23; loop++)
+		for(loop=4; loop<22; loop++)
 			mvwprintw(mywin -> win, loop, 1, linebuf);
 
 		/* display them lines */
 		for(loop=0; loop<cur -> n_strip; loop++)
 		{
-			if (loop == cur_strip) inverse_on(mywin);
+			if (loop == cur_strip) ui_inverse_on(mywin);
 
 			switch((cur -> pstrip)[loop].type)
 			{
@@ -175,7 +176,7 @@ int edit_strippers(void)
 					break;
 			}
 
-			if (loop == cur_strip) inverse_off(mywin);
+			if (loop == cur_strip) ui_inverse_off(mywin);
 			mvwprintw(mywin -> win, 4 + loop, 60, "%d", (cur -> pstrip)[loop].match_count);
 		}
 		mydoupdate();
@@ -191,7 +192,7 @@ int edit_strippers(void)
 				key = c = 'E';
 
 			/* any valid keys? */
-			if (c == 'Q' || c == 7 || c == 'X' || c == 'A' || c == 'E' || c == 'D' || key == KEY_DOWN || key == 13 || key == KEY_UP)
+			if (c == 'Q' || c == abort_key || c == 'X' || c == 'A' || c == 'E' || c == 'D' || key == KEY_DOWN || key == 13 || key == KEY_UP)
 				break;
 
 			if (key == -1) /* T/O waiting for key? then update counters */
@@ -205,7 +206,7 @@ int edit_strippers(void)
 				wrong_key();
 		}
 		/* exit this screen? */
-		if (c == 'Q' || c == 7 || c == 'X')
+		if (c == 'Q' || c == abort_key || c == 'X')
 			break;
 
 		if (key == KEY_UP)
@@ -249,17 +250,17 @@ int edit_strippers(void)
 				c = toupper(wait_for_keypress(HELP_STRIPPER_TYPE, 0, mywin, 0));
 
 				if (c == 'E' || c == 'R' || c == 'C' ||
-						c == 7   || c == 'Q' || c == 'X')
+						c == abort_key   || c == 'Q' || c == 'X')
 					break;
 
 				wrong_key();
 			}
-			if (c == 7 || c == 'Q' || c == 'X') continue;
+			if (c == abort_key || c == 'Q' || c == 'X') continue;
 
 			if (c == 'E')
 			{
 				/* user did not abort edit? */
-				if (edit_stripper_edit_regexp(mywin, &regex_str))
+				if (edit_stripper_edit_regexp(mywin, &regex_str, &case_insensitive))
 					cur_changed = 1;
 			}
 			else if (c == 'R')
@@ -355,7 +356,7 @@ int edit_strippers(void)
 			{
 				if ((cur -> pstrip)[cur_strip].type == STRIP_TYPE_REGEXP)
 				{
-					if (edit_stripper_edit_regexp(mywin, &(cur -> pstrip)[cur_strip].regex_str))
+					if (edit_stripper_edit_regexp(mywin, &(cur -> pstrip)[cur_strip].regex_str, &case_insensitive))
 						changed = 1;
 				}
 				else if ((cur -> pstrip)[cur_strip].type == STRIP_TYPE_RANGE)
@@ -379,19 +380,93 @@ int edit_strippers(void)
 	return changed;
 }
 
+char do_strip_re(regex_t *pre, int *match_count, char *in, char *strip_what)
+{
+	char changed = 0;
+	int search_offset = 0;
+	regmatch_t matches[MAX_N_RE_MATCHES];
+
+	do
+	{
+		int new_offset = -1;
+
+		if (regexec(pre, &in[search_offset], MAX_N_RE_MATCHES, matches, 0) != REG_NOMATCH)
+		{
+			int match_i;
+
+			for(match_i=0; match_i<MAX_N_RE_MATCHES; match_i++)
+			{
+				if (matches[match_i].rm_so == -1 ||
+						matches[match_i].rm_eo == -1)
+					break;
+
+				memset(&strip_what[matches[match_i].rm_so + search_offset], 0x00, matches[match_i].rm_eo - matches[match_i].rm_so);
+				changed = 1;
+
+				new_offset = matches[match_i].rm_eo + search_offset;
+
+				(*match_count)++;
+			}
+		}
+
+		search_offset = new_offset;
+	}
+	while (search_offset != -1);
+
+	return changed;
+}
+
+char do_strip_column(char *delimiter, int col_nr, char *in, char *strip_what)
+{
+	char changed = 0;
+	char *p = in;
+	int del_len = strlen(delimiter);
+	int col_i;
+
+	while(strncmp(p, delimiter, del_len) == 0) p += del_len;
+	if (!*p) return 0;
+	for(col_i=0; col_i<col_nr; col_i++)
+	{
+		p = strstr(p, delimiter);
+		if (!p) break;
+		while(strncmp(p, delimiter, del_len) == 0) p += del_len;
+	}
+	if (!p) return 0;
+
+	changed = 1;
+
+	/* strip the column */
+	while(strncmp(p, delimiter, del_len) != 0 && *p != 0x00)
+	{
+		strip_what[(int)(p - in)] = 0;
+		p++;
+	}
+	/* and strip the delimiter(s) behind it */
+	while(strncmp(p, delimiter, del_len) == 0)
+	{
+		int loop2;
+		for(loop2=0; loop2<del_len; loop2++)
+			strip_what[(int)(p - in) + loop2] = 0;
+		p += del_len;
+	}
+
+	return changed;
+}
+
 char * do_strip(proginfo *cur, char *in)
 {
 	int loop, len;
 	int offset = 0;
 	char *strip_what;
 	char *new_string;
+	char changed = 0;
 
 	if (!in || cur -> n_strip == 0)
-		return mystrdup(in, "do_strip: org string");
+		return NULL;
 
 	len = strlen(in);
 	if (len == 0)
-		return mystrdup(in, "do_strip: org string");
+		return NULL;
 
 	strip_what = (char *)mymalloc(len, "do_strip: strip_what");
 	new_string = (char *)mymalloc(len + 1, "do_strip: new string");
@@ -403,81 +478,36 @@ char * do_strip(proginfo *cur, char *in)
 		if ((cur -> pstrip)[loop].type == STRIP_TYPE_RANGE)
 		{
 			memset(&strip_what[(cur -> pstrip)[loop].start], 0x00, (cur -> pstrip)[loop].end - (cur -> pstrip)[loop].start);
+			changed = 1;
 		}
 		else if ((cur -> pstrip)[loop].type == STRIP_TYPE_REGEXP)
 		{
-			int search_offset = 0;
-			regmatch_t matches[MAX_N_RE_MATCHES];
-
-			do
-			{
-				int new_offset = -1;
-
-				if (regexec(&(cur -> pstrip)[loop].regex, &in[search_offset], MAX_N_RE_MATCHES, matches, 0) != REG_NOMATCH)
-				{
-					int match_i;
-
-					for(match_i=0; match_i<MAX_N_RE_MATCHES; match_i++)
-					{
-						if (matches[match_i].rm_so == -1 ||
-								matches[match_i].rm_eo == -1)
-							break;
-
-						memset(&strip_what[matches[match_i].rm_so + search_offset], 0x00, matches[match_i].rm_eo - matches[match_i].rm_so);
-
-						new_offset = matches[match_i].rm_eo + search_offset;
-
-						(cur -> pstrip)[loop].match_count++;
-					}
-				}
-
-				search_offset = new_offset;
-			}
-			while (search_offset != -1);
+			changed |= do_strip_re(&(cur -> pstrip)[loop].regex, &(cur -> pstrip)[loop].match_count, in, strip_what);
 		}
 		else if ((cur -> pstrip)[loop].type == STRIP_TYPE_COLUMN)
 		{
-			char *p = in;
-			int del_len = strlen((cur -> pstrip)[loop].del);
-			int col_i;
-
-			while(strncmp(p, (cur -> pstrip)[loop].del, del_len) == 0) p += del_len;
-			if (!*p) continue;
-			for(col_i=0; col_i<(cur -> pstrip)[loop].col_nr; col_i++)
-			{
-				p = strstr(p, (cur -> pstrip)[loop].del);
-				if (!p) break;
-				while(strncmp(p, (cur -> pstrip)[loop].del, del_len) == 0) p += del_len;
-			}
-			if (!p) continue;
-
-			/* strip the column */
-			while(strncmp(p, (cur -> pstrip)[loop].del, del_len) != 0 && *p != 0x00)
-			{
-				strip_what[(int)(p - in)] = 0;
-				p++;
-			}
-			/* and strip the delimiter(s) behind it */
-			while(strncmp(p, (cur -> pstrip)[loop].del, del_len) == 0)
-			{
-				int loop2;
-				for(loop2=0; loop2<del_len; loop2++)
-					strip_what[(int)(p - in) + loop2] = 0;
-				p += del_len;
-			}
+			changed |= do_strip_column((cur -> pstrip)[loop].del, (cur -> pstrip)[loop].col_nr, in, strip_what);
 		}
 		else
 			error_exit("do_strip: unknown strip-command detected (%d)\n", (cur -> pstrip)[loop].type);
 	}
 
-	for(loop=0; loop<len; loop++)
+	if (changed)
 	{
-		if (strip_what[loop])
-			new_string[offset++] = in[loop];
+		for(loop=0; loop<len; loop++)
+		{
+			if (strip_what[loop])
+				new_string[offset++] = in[loop];
+		}
+		new_string[offset] = 0x00;
+
+		myfree(strip_what);
+
+		return new_string;
 	}
-	new_string[offset] = 0x00;
 
 	myfree(strip_what);
+	myfree(new_string);
 
-	return new_string;
+	return NULL;
 }
